/**
 * 在孵企业
 */

import moment = require("moment");
import { OPERATIONALDATATYPE, TABLENAME } from "../config/enum/dbEnum";
import { selectData, selectManyTableData } from "../data/operationalData";
import { changeAdd } from "../util/piecemeal";
import { changeEnumValue } from "../util/verificationEnum";
import * as enumConfig from "../config/enum/enum";
import { BizError } from "../util/bizError";
import { ERRORENUM } from "../config/enum/errorEnum";


export async function enterpriseList(entepriseName:string, page:number) {
    let selectParam:any = {};
    if (entepriseName) {
        selectParam.enterpriseName = {"%like%":entepriseName}
    }
    let filesList = ["eId", "enterpriseName", "uscc", "logonTime", "logonAddress", "qiYeGuiMo"];
    let manyTableInfo:any = {};
    manyTableInfo[TABLENAME.租赁信息] = {column:["area", "startTime", "endTime", "roomNumber"], where:{} }; 

    let resInfo = await selectManyTableData(OPERATIONALDATATYPE.多表分页, TABLENAME.企业基础信息表, selectParam, filesList, manyTableInfo, page);

    let dataCount = await selectData(OPERATIONALDATATYPE.查询数据量, TABLENAME.企业基础信息表, selectParam, null);

    let dataList = [];
    resInfo.forEach(info => {
        let {eId, enterpriseName, uscc, logonTime, logonAddress, qiYeGuiMo, enterprise_leases} = info;
        let leasesTime = '-'
        if (enterprise_leases[0].startTime){
            leasesTime = `${moment(enterprise_leases[0].startTime).format("YYYY-MM-DD")}至${moment(enterprise_leases[0].endTime).format("YYYY-MM-DD")}`;
        }

        let dataInfo = {
            eId,
            enterpriseName, //企业名称
            uscc, //统一信用代码
            logonTime:moment(logonTime).format("YYYY-MM-DD"), //注册日期
            logonAddress:changeAdd(JSON.parse(logonAddress)), //注册地址
            qiYeGuiMo,//企业规模
            leasesTime,//租赁时间
            area:enterprise_leases[0].area+"㎡",//租赁面积
            roomNumber:enterprise_leases[0].roomNumber,//室号
        };
        dataList.push(dataInfo);
    })
    

    return {dataList, dataCount};
}


/**
 * 在孵企业详情
 * @param eId 
 */
export async function getDetails(eId:string) {
    if (!eId) throw new BizError(ERRORENUM.参数错误, `在孵企业详细信息缺失eId`)

    let details = {
        enterprise: await enterpriseById(eId),
        manage: await enterpriseManageById(eId),
        financing: await enterpriseFinancingById(eId),
        talent: await enterpriseTalentById(eId),
        service: await enterpriseServiceById(eId),
    }

    return details;
}




/**
 * 在孵企业详情 基础信息
 * @param eId 
 * @returns 
 */
export async function enterpriseById(eId:string) {
    let filesList = ["enterpriseName", "uscc", "logonTime", "logonAddress", "qiYeGuiMo"];
    let manyTableInfo:any = {};
    manyTableInfo[TABLENAME.租赁信息] = {column:["area", "startTime", "endTime", "roomNumber"], where:{eId} }; 

    let resInfo = await selectManyTableData(OPERATIONALDATATYPE.多表联查, TABLENAME.企业基础信息表, {eId}, filesList, manyTableInfo);

    let dataList = [];
    resInfo.forEach(info => {
        let {enterpriseName, uscc, logonTime, logonAddress, qiYeGuiMo, enterprise_leases} = info;
        let leasesTime = '-'
        if (enterprise_leases[0].startTime){
            leasesTime = `${moment(enterprise_leases[0].startTime).format("YYYY-MM-DD")}至${moment(enterprise_leases[0].endTime).format("YYYY-MM-DD")}`;
        }

        let dataInfo = [
            {
                key:"企业名称",
                value:enterpriseName
            },
            {
                key:"统一信用代码",
                value:uscc
            },
            {
                key:"注册日期",
                value:moment(logonTime).format("YYYY-MM-DD")
            },
            {
                key:"注册地址",
                value:changeAdd(JSON.parse(logonAddress))
            },
            {
                key:"企业规模",
                value:qiYeGuiMo
            },
            {
                key:"租赁时间",
                value:leasesTime
            },
            {
                key:"租赁面积",
                value:enterprise_leases[0].area+"㎡"
            },
            {
                key:"室号",
                value:enterprise_leases[0].roomNumber
            }
        ];
        dataList.push(dataInfo);
    })

    return dataList;
}


/**
 * 在孵企业详情 经营数据
 * @param eId 
 * @returns 
 */
export async function enterpriseManageById(eId:string) {
    let manyTableInfo:any = {};
    manyTableInfo[TABLENAME.企业基础信息表] = {column: ["enterpriseName", "uscc"], where:{eId} };
    let resInfo = await selectManyTableData(OPERATIONALDATATYPE.多表联查, TABLENAME.企业经营信息, {eId}, ["mId", "BI", "TXP", "RD", "annual", "state"], manyTableInfo);
    let resData = [];
    resInfo.forEach( info => {
        let {BI, mId, TXP,RD,state,annual, enterprise} = info;
        resData.push({BI, mId, TXP,RD,state,annual:moment(annual).format("YYYY"), enterprise});
    })
    
    resData.sort( (a, b) => {
        return b.annual - a.annual;
    })

    let dataList = [];
    resData.forEach(info => {
        let {BI, mId, TXP,RD,state,annual, enterprise} = info;

        let dataInfo = [
            {
                key:"企业名称",
                value:enterprise.enterpriseName
            },
            {
                key:"统一信用代码",
                value:enterprise.uscc
            },
            {
                key:"营收（万元）",
                value:BI
            },
            {
                key:"纳税（万元）",
                value:TXP
            },
            {
                key:"研发投入（万元）",
                value:RD
            },
            {
                key:"年度",
                value:annual
            },
            {
                key:"审核状态",
                value:state ? "已审核" : "未审核"
            }
        ]
        dataList.push(dataInfo);
    });

    return dataList;
}


/**
 * 在孵企业详情 融资情况
 * @param eId 
 * @returns 
 */
export async function enterpriseFinancingById(eId:string) {
    let manyTableInfo:any = {};
    manyTableInfo[TABLENAME.企业融资] = {column:["financingAmount", "financingRounds", "investmentDate", "investmentInstitutionsName" ], where:{eId} };
    let filesList = ["enterpriseName", "uscc"];
    let resInfo = await selectManyTableData(OPERATIONALDATATYPE.多表联查, TABLENAME.企业基础信息表, {eId}, filesList, manyTableInfo);
    
    let resData = [];
    resInfo.forEach( info => {
        let {enterpriseName, uscc, enterprise_financings} = info;
        resData.push({enterpriseName, uscc, enterprise_financings, investmentDate:new Date(enterprise_financings[0].investmentDate).valueOf()});
    })
    
    resData.sort( (a, b) => {
        return b.investmentDate - a.investmentDate;
    })

    let dataList = [];
    resData.forEach(info => {
        let {enterpriseName, uscc, enterprise_financings} = info;

        let dataInfo = [
            {
                key:"企业名称",
                value:enterpriseName
            },
            {
                key:"统一信用代码",
                value:uscc
            },
            {
                key:"融资金额（万元）",
                value:enterprise_financings[0].financingAmount
            },
            {
                key:"融资轮次",
                value:changeEnumValue(enumConfig.FINANCINGROUNDS, enterprise_financings[0].financingRounds)
            },
            {
                key:"获得投资时间",
                value:moment(enterprise_financings[0].investmentDate).format("YYYY-MM-DD")
            },
            {
                key:"投资机构名称",
                value:enterprise_financings[0].investmentInstitutionsName
            },
        ]
        dataList.push(dataInfo);
    });

    return dataList;
}


/**
 * 在孵企业详情 创始团队
 * @param eId 
 * @returns 
 */
export async function enterpriseTalentById(eId:string) {
    let manyTableInfo:any = {};
    manyTableInfo[TABLENAME.企业基础信息表] = {column:["enterpriseName" ], where:{eId} };
    let filesList = ["name", "sex", "birth", "degree", "graduationSchool"];
    let resInfo = await selectManyTableData(OPERATIONALDATATYPE.多表联查, TABLENAME.创业团队, {eId}, filesList, manyTableInfo);

    let dataList = [];
    resInfo.forEach(info => {
        let {name, sex, birth, degree, graduationSchool, enterprise} = info;

        let dataInfo = [
            {
                key:"姓名",
                value:name
            },
            {
                key:"性别",
                value:changeEnumValue(enumConfig.SEX, sex)
            },
            {
                key:"出生年月",
                value:moment(birth).format("YYYY-MM-DD")
            },
            {
                key:"最高学历",
                value:changeEnumValue(enumConfig.DEGREE, degree)
            },
            {
                key:"毕业学校",
                value:graduationSchool
            },
            {
                key:"企业名称",
                value:enterprise.enterpriseName
            },
        ]

        dataList.push(dataInfo);
    });

    return dataList;
}


/**
 * 在孵企业详情 企业服务
 * @param eId 
 * @returns 
 */
export async function enterpriseServiceById(eId:string) {
    let filesList = ["esId", "needCategory", "applyTime", "followUpStatus", "resolveTime", "needContent"];
    let manyTableInfo:any = {};
    manyTableInfo[TABLENAME.企业基础信息表] = {column:["eId",  "enterpriseName"], where:{eId} };
    let resList = await selectManyTableData(OPERATIONALDATATYPE.多表联查, TABLENAME.企业服务表, {eId}, filesList, manyTableInfo );

    let resData = [];
    resList.forEach( info => {
        let {enterprise, needCategory, applyTime, followUpStatus, resolveTime, needContent} = info;
        resData.push({enterprise, needCategory, applyTime:new Date(applyTime).valueOf(), followUpStatus, resolveTime, needContent});
    })
    
    resData.sort( (a, b) => {
        return b.applyTime - a.applyTime;
    })

    let dataList = [];
    resData.forEach(info => {
        let dataInfo = [
            {
                key:"企业名称",
                value:info.enterprise.enterpriseName
            },
            {
                key:"需求类别",
                value:changeEnumValue(enumConfig.NEEDCATEGORY, info.needCategory)
            },
            {
                key:"申请时间",
                value:moment(info.applyTime).format("YYYY-MM-DD HH:mm")
            },
            {
                key:"跟进状态",
                value:changeEnumValue(enumConfig.FOLLOWUPSTATUS, info.followUpStatus)
            },
            {
                key:"解决时间",
                value:info.resolveTime ? moment(info.resolveTime).format("YYYY-MM-DD HH:mm") : '-'
            },
            {
                key:"需求内容",
                value:info.needContent
            },
        ]

        dataList.push(dataInfo);
    });

    return dataList;
}



//下载
export async function dwEnterpriseTable(entepriseName:string, type:number, files) {
    
    let selectParam:any = {};
    if (type ==1) {
        if (entepriseName) {
            selectParam.enterpriseName = {"%like%":entepriseName}
        }
    }
    
    let filesList = ["enterpriseName", "uscc", "logonTime", "logonAddress", "qiYeGuiMo"];
    let manyTableInfo:any = {};
    manyTableInfo[TABLENAME.租赁信息] = {column:["area", "startTime", "endTime", "roomNumber"], where:{} }; 

    let resInfo = await selectManyTableData(OPERATIONALDATATYPE.多表联查, TABLENAME.企业基础信息表, selectParam, filesList, manyTableInfo);

    let dataList = [];
    let titleList = []
    let valueList = [];
    files.forEach(item => {
        titleList.push(item.key);
        valueList.push(item.value);
    });
    dataList.push(titleList);
    resInfo.forEach(info => {
        let {enterpriseName, uscc, logonTime, logonAddress, qiYeGuiMo, enterprise_leases} = info;
        let leasesTime = '-'
        if (enterprise_leases[0].startTime){
            leasesTime = `${moment(enterprise_leases[0].startTime).format("YYYY-MM-DD")}至${moment(enterprise_leases[0].endTime).format("YYYY-MM-DD")}`;
        }
        let subList = [];
        valueList.forEach(subInfo => {
            if (subInfo == "enterpriseName") subList.push(enterpriseName);//企业名称
            if (subInfo == "uscc") subList.push(uscc); //统一信用代码
            if (subInfo == "logonTime") subList.push(moment(logonTime).format("YYYY-MM-DD"));//注册日期
            if (subInfo == "logonAddress") subList.push(changeAdd(JSON.parse(logonAddress)));//注册地址
            if (subInfo == "qiYeGuiMo") subList.push(qiYeGuiMo);//企业规模
            if (subInfo == "startTime") subList.push(leasesTime);//孵化时间
            if (subInfo == "area") subList.push(enterprise_leases[0].area+"㎡");//租赁面积
            if (subInfo == "roomNumber") subList.push(enterprise_leases[0].roomNumber);//室号
        });
       

        dataList.push(subList);
    })
    

    return {dataList};
}